@@ -6,7 +6,7 @@
    see the file COPYING for details
    or visit http://www.gnu.org/copyleft/gpl.html
 
-   $Id: common_memory.cpp 3197 2006-03-02 19:50:15Z mosu $
+   $Id: common_memory.cpp 3396 2006-12-30 13:17:42Z mosu $
 
    class definition for memory handling classes
 
@@ -28,15 +28,18 @@ memory_c::resize(int new_size) throw() {
   if (!its_counter)
     its_counter = new counter(NULL, 0, false);
 
-  if (its_counter->is_free)
-    its_counter->ptr = (X *)saferealloc(its_counter->ptr, new_size);
-  else {
+  if (its_counter->is_free) {
+    its_counter->ptr = (X *)saferealloc(its_counter->ptr,
+                                        new_size + its_counter->offset);
+    its_counter->size = new_size + its_counter->offset;
+  } else {
     X *tmp = (X *)safemalloc(new_size);
-    memcpy(tmp, its_counter->ptr, its_counter->size);
+    memcpy(tmp, its_counter->ptr + its_counter->offset,
+           its_counter->size - its_counter->offset);
     its_counter->ptr = tmp;
     its_counter->is_free = true;
+    its_counter->size = new_size;
   }
-  its_counter->size = new_size;
 }
 
 memory_cptr
