@@ -6,7 +6,7 @@
    see the file COPYING for details
    or visit http://www.gnu.org/copyleft/gpl.html
 
-   $Id: xml_element_parser.cpp 3115 2005-10-12 16:41:09Z mosu $
+   $Id: xml_element_parser.cpp 3416 2007-01-03 14:24:06Z mosu $
 
    XML element parser
 
@@ -300,25 +300,28 @@ add_new_element(parser_data_t *pdata,
                pdata->mapping[parent_idx].name);
 
   if (pdata->depth > 0) {
-    const EbmlSemanticContext &context =
+    const EbmlCallbacks *callbacks =
       find_ebml_callbacks(KaxSegment::ClassInfos,
-                          pdata->mapping[parent_idx].id).Context;
+                          pdata->mapping[parent_idx].id);
     found = false;
-    for (i = 0; i < context.Size; i++)
-      if (pdata->mapping[elt_idx].id ==
-          context.MyTable[i].GetCallbacks.GlobalId) {
-        found = true;
-        break;
-      }
+    if (NULL != callbacks) {
+      const EbmlSemanticContext &context = callbacks->Context;
+      for (i = 0; i < context.Size; i++)
+        if (pdata->mapping[elt_idx].id ==
+            context.MyTable[i].GetCallbacks.GlobalId) {
+          found = true;
+          break;
+        }
+    }
 
     if (!found)
       xmlp_error(pdata, "<%s> is not a valid child element of <%s>.", name,
                  pdata->mapping[parent_idx].name);
 
-    const EbmlSemantic &semantic =
+    const EbmlSemantic *semantic =
       find_ebml_semantic(KaxSegment::ClassInfos,
                          pdata->mapping[elt_idx].id);
-    if (semantic.Unique) {
+    if ((NULL != semantic) && semantic->Unique) {
       m = dynamic_cast<EbmlMaster *>(xmlp_pelt);
       assert(m != NULL);
       for (i = 0; i < m->ListSize(); i++)
@@ -520,8 +523,8 @@ parse_xml_elements(const char *parser_name,
       if (XML_Parse(parser, buffer.c_str(), buffer.length(), done) == 0) {
         xerror = XML_GetErrorCode(parser);
         error = mxsprintf("XML parser error at line %d of '%s': %s. ",
-                          XML_GetCurrentLineNumber(parser), pdata->file_name,
-                          XML_ErrorString(xerror));
+                          (int)XML_GetCurrentLineNumber(parser),
+                          pdata->file_name, XML_ErrorString(xerror));
         if (xerror == XML_ERROR_INVALID_TOKEN)
           error += "Remember that special characters like &, <, > and \" "
             "must be escaped in the usual HTML way: &amp; for '&', "
