@@ -5,7 +5,7 @@
    see the file COPYING for details
    or visit http://www.gnu.org/copyleft/gpl.html
 
-   $Id: xtr_base.cpp 3177 2006-02-03 09:25:30Z mosu $
+   $Id: xtr_base.cpp 3461 2007-01-12 14:44:59Z mosu $
 
    extracts tracks from Matroska files into other files
 
@@ -28,6 +28,7 @@
 #include "xtr_avi.h"
 #include "xtr_base.h"
 #include "xtr_cpic.h"
+#include "xtr_mpeg1_2.h"
 #include "xtr_ogg.h"
 #include "xtr_rmff.h"
 #include "xtr_textsubs.h"
@@ -40,9 +41,11 @@ using namespace libmatroska;
 
 xtr_base_c::xtr_base_c(const string &_codec_id,
                        int64_t _tid,
-                       track_spec_t &tspec):
-  codec_id(_codec_id), file_name(tspec.out_name), master(NULL), out(NULL),
-  tid(_tid), default_duration(0), bytes_written(0) {
+                       track_spec_t &tspec,
+                       const char *_container_name):
+  codec_id(_codec_id), file_name(tspec.out_name),
+  container_name(NULL == _container_name ? "raw data" : _container_name),
+  master(NULL), out(NULL), tid(_tid), default_duration(0), bytes_written(0) {
 }
 
 xtr_base_c::~xtr_base_c() {
@@ -105,10 +108,14 @@ xtr_base_c::create_extractor(const string &new_codec_id,
   else if (2 == tspec.extract_raw)
     return new xtr_fullraw_c(new_codec_id, new_tid, tspec);
   // Audio formats
-  else if ((new_codec_id == MKV_A_AC3) ||
-           starts_with_case(new_codec_id, "A_MPEG/L") ||
-           (new_codec_id == MKV_A_DTS))
-    return new xtr_base_c(new_codec_id, new_tid, tspec);
+  else if (new_codec_id == MKV_A_AC3)
+    return new xtr_base_c(new_codec_id, new_tid, tspec, "Dolby Digital (AC3)");
+  else if (starts_with_case(new_codec_id, "A_MPEG/L"))
+    return new xtr_base_c(new_codec_id, new_tid, tspec,
+                          "MPEG-1 Audio Layer 2/3");
+  else if (new_codec_id == MKV_A_DTS)
+    return new xtr_base_c(new_codec_id, new_tid, tspec,
+                          "Digital Theater System (DTS)");
   else if (new_codec_id == MKV_A_PCM)
     return new xtr_wav_c(new_codec_id, new_tid, tspec);
   else if (new_codec_id == MKV_A_FLAC) {
@@ -134,6 +141,9 @@ xtr_base_c::create_extractor(const string &new_codec_id,
     return new xtr_avc_c(new_codec_id, new_tid, tspec);
   else if (starts_with_case(new_codec_id, "V_REAL/"))
     return new xtr_rmff_c(new_codec_id, new_tid, tspec);
+  else if ((new_codec_id == MKV_V_MPEG1) ||
+           (new_codec_id == MKV_V_MPEG2))
+    return new xtr_mpeg1_2_video_c(new_codec_id, new_tid, tspec);
 
   // Subtitle formats
   else if ((new_codec_id == MKV_S_TEXTUTF8) ||
