@@ -5,7 +5,7 @@
    see the file COPYING for details
    or visit http://www.gnu.org/copyleft/gpl.html
 
-   $Id: xtr_rmff.h 3126 2005-10-19 16:52:55Z mosu $
+   $Id: xtr_rmff.h 3461 2007-01-12 14:44:59Z mosu $
 
    extracts tracks from Matroska files into other files
 
@@ -36,6 +36,10 @@ public:
                             bool references_valid);
   virtual void finish_file();
   virtual void headers_done();
+
+  virtual const char *get_container_name() {
+    return "RMFF (RealMedia File Format)";
+  };
 };
 
 #endif
