@@ -5,7 +5,7 @@
    see the file COPYING for details
    or visit http://www.gnu.org/copyleft/gpl.html
 
-   $Id: mkvinfo.cpp 3131 2005-10-21 20:57:23Z mosu $
+   $Id: mkvinfo.cpp 3455 2007-01-10 13:13:23Z mosu $
 
    retrieves and displays information about a Matroska file
 
@@ -25,7 +25,7 @@
 #include <time.h>
 
 #if defined(COMP_MSC)
-#include <assert.h>
+#include <cassert>
 #else
 #include <unistd.h>
 #endif
@@ -98,6 +98,7 @@ int num_tracks = 0;
 bool use_gui = false;
 bool calc_checksums = false;
 bool show_summary = false;
+bool show_hexdump = false;
 uint64_t tc_scale = TIMECODE_SCALE;
 
 void
@@ -132,7 +133,6 @@ find_track_by_uid(int tuid) {
 
 void
 set_usage() {
-#ifdef HAVE_WXWINDOWS
   usage_text = Y(
     "Usage: mkvinfo [options] inname\n\n"
     " options:\n"
@@ -142,28 +142,13 @@ set_usage() {
     "                 description of what mkvinfo outputs.\n"
     "  -c, --checksum Calculate and display checksums of frame contents.\n"
     "  -s, --summary  Only show summaries of the contents, not each element.\n"
+    "  -x, --hexdump  Show the first 16 bytes of each frame as a hex dump.\n"
     "  --output-charset <charset>\n"
     "                 Output messages in this charset\n"
     "  -r, -o, --redirect-output file.ext\n"
     "                 Redirect all messages to this file.\n"
     "  -h, --help     Show this help.\n"
     "  -V, --version  Show version information.\n");
-#else
-  usage_text = Y(
-    "Usage: mkvinfo [options] inname\n\n"
-    " options:\n"
-    "  inname         Use 'inname' as the source.\n"
-    "  -v, --verbose  Increase verbosity. See the man page for a detailed\n"
-    "                 description of what mkvinfo outputs.\n"
-    "  -c, --checksum Calculate and display checksums of frame contents.\n"
-    "  -s, --summary  Only show summaries of the contents, not each element.\n"
-    "  --output-charset <charset>\n"
-    "                 Output messages in this charset\n"
-    "  -r, -o, --redirect-output file.ext\n"
-    "                 Redirect all messages to this file.\n"
-    "  -h, --help     Show this help.\n"
-    "  -V, --version  Show version information.\n");
-#endif
 
   version_info = "mkvinfo v" VERSION " ('" VERSIONNAME "')";
 }
@@ -185,12 +170,7 @@ show_error(const char *fmt,
   vsnprintf(args_buffer, ARGS_BUFFER_LEN - 1, new_fmt.c_str(), ap);
   va_end(ap);
 
-#ifdef HAVE_WXWINDOWS
-  if (use_gui)
-    frame->show_error(wxU(args_buffer));
-  else
-#endif
-    mxinfo("(%s) %s\n", NAME, args_buffer);
+  ui_show_error(args_buffer);
 }
 
 #define show_warning(l, f, args...) _show_element(NULL, NULL, false, l, f, \
@@ -238,27 +218,10 @@ _show_element(EbmlElement *l,
   vsnprintf(args_buffer, ARGS_BUFFER_LEN - 1, new_fmt.c_str(), ap);
   va_end(ap);
 
-  if (!use_gui) {
-    char *level_buffer;
-
-    level_buffer = new char[level + 1];
-    memset(&level_buffer[1], ' ', level);
-    level_buffer[0] = '|';
-    level_buffer[level] = 0;
-    mxinfo("%s+ %s", level_buffer, args_buffer);
-    if ((verbose > 1) && (l != NULL))
-      mxinfo(" at " LLU, l->GetElementPosition());
-    mxinfo("\n");
-    delete []level_buffer;
-  }
-#ifdef HAVE_WXWINDOWS
-  else {
-    if (l != NULL)
-      mxprints(&args_buffer[strlen(args_buffer)], " at " LLU,
-               l->GetElementPosition());
-    frame->add_item(level, wxU(args_buffer));
-  }
-#endif // HAVE_WXWINDOWS
+  int64_t pos = -1;
+  if (NULL != l)
+    pos = l->GetElementPosition();
+  ui_show_element(level, args_buffer, pos);
 
   if ((l != NULL) && skip) {
     // Dump unknown elements recursively.
@@ -270,6 +233,18 @@ _show_element(EbmlElement *l,
   }
 }
 
+string
+create_hexdump(const unsigned char *buf,
+               int size) {
+  string hex(" hexdump");
+  int bmax = size >= 16 ? 16 : size, b;
+
+  for (b = 0; b < bmax; ++b)
+    hex += mxsprintf(" %02x", buf[b]);
+
+  return hex;
+}
+
 // }}}
 
 // {{{ FUNCTION parse_args
@@ -289,11 +264,9 @@ parse_args(vector<string> args,
   // Now parse the rest of the arguments.
   for (i = 0; i < args.size(); i++)
     if ((args[i] == "-g") || (args[i] == "--gui")) {
-#ifndef HAVE_WXWINDOWS
-      mxerror("mkvinfo was compiled without GUI support.\n");
-#else // HAVE_WXWINDOWS
+      if (!ui_graphical_available())
+        mxerror("mkvinfo was compiled without GUI support.\n");
       use_gui = true;
-#endif // HAVE_WXWINDOWS
     } else if ((args[i] == "-c") || (args[i] == "--checksum"))
       calc_checksums = true;
     else if ((args[i] == "-C") || (args[i] == "--check-mode")) {
@@ -302,7 +275,9 @@ parse_args(vector<string> args,
     } else if ((args[i] == "-s") || (args[i] == "--summary")) {
       calc_checksums = true;
       show_summary = true;
-    } else if (file_name != "")
+    } else if ((args[i] == "-x") || (args[i] == "--hexdump"))
+      show_hexdump = true;
+    else if (file_name != "")
       mxerror("Only one input file is allowed.\n");
     else
       file_name = args[i];
@@ -1071,6 +1046,9 @@ def_handle(tracks) {
             fourcc_buffer +=
               mxsprintf(" (adler: 0x%08x)",
                         calc_adler32(c_priv.GetBuffer(), c_priv.GetSize()));
+          if (show_hexdump)
+            fourcc_buffer += create_hexdump(c_priv.GetBuffer(),
+                                            c_priv.GetSize());
           show_element(l3, 3, "CodecPrivate, length %d%s",
                        (int)c_priv.GetSize(), fourcc_buffer.c_str());
 
@@ -1463,6 +1441,7 @@ def_handle2(block_group,
   int i2, i3, i4, i;
   vector<int> frame_sizes;
   vector<uint32_t> frame_adlers;
+  vector<string> frame_hexdumps;
   bool fref_found, bref_found;
   uint64_t lf_tnum, lf_timecode;
   float bduration;
@@ -1495,16 +1474,20 @@ def_handle2(block_group,
       bduration = -1.0;
       for (i = 0; i < (int)block.NumberFrames(); i++) {
         DataBuffer &data = block.GetBuffer(i);
+        string hex;
         if (calc_checksums)
           mxprints(adler, " (adler: 0x%08x)",
                    calc_adler32(data.Buffer(), data.Size()));
         else
           adler[0] = 0;
-        show_element(NULL, 4, "Frame with size %u%s", data.Size(),
-                     adler);
+        if (show_hexdump)
+          hex = create_hexdump(data.Buffer(), data.Size());
+        show_element(NULL, 4, "Frame with size %u%s%s", data.Size(),
+                     adler, hex.c_str());
         frame_sizes.push_back(data.Size());
         frame_adlers.push_back(calc_adler32(data.Buffer(),
                                             data.Size()));
+        frame_hexdumps.push_back(hex);
       }
 
     } else if (is_id(l3, KaxBlockDuration)) {
@@ -1659,18 +1642,20 @@ def_handle2(block_group,
     for (fidx = 0; fidx < frame_sizes.size(); fidx++) {
       if (bduration != -1.0)
         mxinfo(Y("%c frame, track " LLU ", timecode " LLD " (" FMT_TIMECODE
-                 "), duration %.3f, size %d, adler 0x%08x\n"),
+                 "), duration %.3f, size %d, adler 0x%08x%s\n"),
                bref_found && fref_found ? 'B' :
                bref_found ? 'P' : !fref_found ? 'I' : 'P',
                lf_tnum, lf_timecode, ARG_TIMECODE(lf_timecode),
-               bduration, frame_sizes[fidx], frame_adlers[fidx]);
+               bduration, frame_sizes[fidx], frame_adlers[fidx],
+               frame_hexdumps[fidx].c_str());
       else
         mxinfo(Y("%c frame, track " LLU ", timecode " LLD " (" FMT_TIMECODE
-                 "), size %d, adler 0x%08x\n"),
+                 "), size %d, adler 0x%08x%s\n"),
                bref_found && fref_found ? 'B' :
                bref_found ? 'P' : !fref_found ? 'I' : 'P',
                lf_tnum, lf_timecode, ARG_TIMECODE(lf_timecode),
-               frame_sizes[fidx], frame_adlers[fidx]);
+               frame_sizes[fidx], frame_adlers[fidx],
+               frame_hexdumps[fidx].c_str());
     }
   } else if (verbose > 2)
     show_element(NULL, 2, Y("[%c frame for track " LLU ", timecode " LLD "]"),
@@ -1706,12 +1691,16 @@ def_handle2(simple_block,
   bduration = -1.0;
   for (i = 0; i < (int)block.NumberFrames(); i++) {
     DataBuffer &data = block.GetBuffer(i);
+    string hex;
     if (calc_checksums)
       mxprints(adler, " (adler: 0x%08x)",
                calc_adler32(data.Buffer(), data.Size()));
     else
       adler[0] = 0;
-    show_element(NULL, 4, "Frame with size %u%s", data.Size(), adler);
+    if (show_hexdump)
+      hex = create_hexdump(data.Buffer(), data.Size());
+    show_element(NULL, 4, "Frame with size %u%s%s", data.Size(), adler,
+                 hex.c_str());
     frame_sizes.push_back(data.Size());
     frame_adlers.push_back(calc_adler32(data.Buffer(),
                                         data.Size()));
@@ -1742,11 +1731,9 @@ def_handle2(cluster,
 
   cluster = (KaxCluster *)l1;
 
-#ifdef HAVE_WXWINDOWS
   if (use_gui)
-    frame->show_progress(100 * cluster->GetElementPosition() /
-                         file_size, wxT("Parsing file"));
-#endif // HAVE_WXWINDOWS
+    ui_show_progress(100 * cluster->GetElementPosition() / file_size,
+                     "Parsing file");
 
   upper_lvl_el = 0;
   m1 = static_cast<EbmlMaster *>(l1);
@@ -1984,8 +1971,7 @@ process_file(const string &file_name) {
         if (l0->GetSize() == -1)
           show_element(l0, 0, "Segment, size unknown");
         else
-          show_element(l0, 0, "Segment, size " LLD, l0->GetSize() -
-                       l0->HeadSize());
+          show_element(l0, 0, "Segment, size " LLD, l0->GetSize());
         break;
       }
 
@@ -2127,17 +2113,6 @@ console_main(vector<string> args) {
     return 1;
 }
 
-#if !defined HAVE_WXWINDOWS
-int
-main(int argc,
-     char **argv) {
-  setup();
-
-  return console_main(command_line_utf8(argc, argv));
-}
-
-#elif defined(SYS_UNIX) || defined(SYS_APPLE)
-
 int
 main(int argc,
      char **argv) {
@@ -2149,11 +2124,8 @@ main(int argc,
   args = command_line_utf8(argc, argv);
   parse_args(args, initial_file);
 
-  if (use_gui) {
-    wxEntry(argc, argv);
-    return 0;
-  } else
+  if (use_gui)
+    return ui_run(argc, argv);
+  else
     return console_main(args);
 }
-
-#endif // HAVE_WXWINDOWS
