@@ -6,7 +6,7 @@
    see the file COPYING for details
    or visit http://www.gnu.org/copyleft/gpl.html
 
-   $Id: r_avi.cpp 3141 2005-11-12 17:41:01Z mosu $
+   $Id: r_avi.cpp 3464 2007-01-13 17:41:08Z mosu $
 
    AVI demultiplexer module
 
@@ -36,6 +36,7 @@ extern "C" {
 #include "r_avi.h"
 #include "p_aac.h"
 #include "p_ac3.h"
+#include "p_avc.h"
 #include "p_dts.h"
 #include "p_mp3.h"
 #include "p_pcm.h"
@@ -141,12 +142,22 @@ avi_reader_c::create_packetizer(int64_t tid) {
         !strcasecmp(codec, "MPG3") ||
         !strcasecmp(codec, "MP43"))
       divx_type = DIVX_TYPE_V3;
-    else if (is_mpeg4_p2_fourcc(codec))
+    else if (mpeg4::p2::is_fourcc(codec))
       divx_type = DIVX_TYPE_MPEG4;
 
     ti.private_data = (unsigned char *)avi->bitmap_info_header;
     if (ti.private_data != NULL)
       ti.private_size = get_uint32_le(&avi->bitmap_info_header->bi_size);
+
+    mxverb(4, "track extra data size: %d\n",
+           (int)(ti.private_size - sizeof(alBITMAPINFOHEADER)));
+    if (sizeof(alBITMAPINFOHEADER) < ti.private_size) {
+      mxverb(4, "  ");
+      for (i = sizeof(alBITMAPINFOHEADER); i < ti.private_size; ++i)
+        mxverb(4, "%02x ", ti.private_data[i]);
+      mxverb(4, "\n");
+    }
+
     ti.id = 0;                 // ID for the video track.
     if (divx_type == DIVX_TYPE_MPEG4) {
       vptzr =
@@ -159,6 +170,25 @@ avi_reader_c::create_packetizer(int64_t tid) {
       if (verbose)
         mxinfo(FMT_TID "Using the MPEG-4 part 2 video output module.\n",
                ti.fname.c_str(), (int64_t)0);
+    } else if (mpeg4::p10::is_avc_fourcc(codec)) {
+      try {
+        memory_cptr avcc = extract_avcc();
+        mpeg4_p10_es_video_packetizer_c *ptzr =
+          new mpeg4_p10_es_video_packetizer_c(this, avcc, AVI_video_width(avi),
+                                              AVI_video_height(avi), ti);
+        vptzr = add_packetizer(ptzr);
+        ptzr->enable_timecode_generation(false);
+        ptzr->set_track_default_duration((int64_t)(1000000000 /
+                                                   AVI_frame_rate(avi)));
+        if (verbose)
+          mxinfo(FMT_TID "Using the MPEG-4 part 10 ES video output module.\n",
+                 ti.fname.c_str(), (int64_t)0);
+
+      } catch (...) {
+        mxerror(FMT_TID "Could not extract the decoder specific config data "
+                "(AVCC) from this AVC/h.264 track.\n",
+                ti.fname.c_str(), (int64_t)0);
+      }
     } else {
       vptzr = add_packetizer(new video_packetizer_c(this, NULL,
                                                     AVI_frame_rate(avi),
@@ -187,6 +217,40 @@ avi_reader_c::create_packetizers() {
     create_packetizer(i + 1);
 }
 
+memory_cptr
+avi_reader_c::extract_avcc() {
+  avc_es_parser_c parser;
+  int i, size, key;
+
+  parser.ignore_nalu_size_length_errors();
+  if (map_has_key(ti.nalu_size_lengths, 0))
+    parser.set_nalu_size_length(ti.nalu_size_lengths[0]);
+  else if (map_has_key(ti.nalu_size_lengths, -1))
+    parser.set_nalu_size_length(ti.nalu_size_lengths[-1]);
+
+  for (i = 0; i < max_video_frames; ++i) {
+    size = AVI_frame_size(avi, i);
+    if (0 == size)
+      continue;
+
+    memory_cptr buffer(new memory_c((unsigned char *)safemalloc(size),
+                                    size, true));
+    AVI_set_video_position(avi, i);
+    size = AVI_read_frame(avi, (char *)buffer->get(), &key);
+    if (0 < size) {
+      parser.add_bytes(buffer->get(), size);
+      if (parser.headers_parsed()) {
+        AVI_set_video_position(avi, 0);
+        return parser.get_avcc();
+      }
+    }
+  }
+
+  AVI_set_video_position(avi, 0);
+
+  throw false;
+}
+
 // {{{ FUNCTION avi_reader_c::add_audio_demuxer
 
 void
@@ -198,7 +262,7 @@ avi_reader_c::add_audio_demuxer(int aid) {
   uint32_t audio_format;
   int maxchunks, i;
 
-  foreach(it, ademuxers)
+  mxforeach(it, ademuxers)
     if (it->aid == aid) // Demuxer already added?
       return;
 
@@ -468,7 +532,7 @@ avi_reader_c::read(generic_packetizer_c *ptzr,
   if ((vptzr != -1) && (PTZR(vptzr) == ptzr))
     return read_video();
 
-  foreach(demuxer, ademuxers)
+  mxforeach(demuxer, ademuxers)
     if ((-1 != demuxer->ptzr) && (PTZR(demuxer->ptzr) == ptzr))
       return read_audio(*demuxer);
 
@@ -511,7 +575,7 @@ avi_reader_c::identify() {
     if (size > 0) {
       buffer = (unsigned char *)safemalloc(size);
       AVI_read_frame(avi, (char *)buffer, &key);
-      if (mpeg4_p2_extract_par(buffer, size, par_num, par_den)) {
+      if (mpeg4::p2::extract_par(buffer, size, par_num, par_den)) {
         width = AVI_video_width(avi);
         height = AVI_video_height(avi);
         aspect_ratio = (float)width / (float)height * (float)par_num /
@@ -556,7 +620,7 @@ avi_reader_c::identify() {
         type = "AAC";
         break;
       default:
-        type = "unknown";
+        type = "unsupported";
     }
     mxinfo("Track ID %d: audio (%s)\n", i + 1, type);
   }
