@@ -6,7 +6,7 @@
    see the file COPYING for details
    or visit http://www.gnu.org/copyleft/gpl.html
 
-   $Id: r_real.cpp 3143 2005-11-12 18:04:03Z mosu $
+   $Id: r_real.cpp 3395 2006-12-30 12:56:33Z mosu $
 
    RealMedia demultiplexer module
 
@@ -209,9 +209,7 @@ real_reader_c::parse_headers() {
       ts_data = track->mdpr_header.type_specific_data;
       ts_size = get_uint32_be(&track->mdpr_header.type_specific_size);
 
-      real_demuxer_cptr dmx(new real_demuxer_t);
-      dmx->track = track;
-      dmx->ptzr = -1;
+      real_demuxer_cptr dmx(new real_demuxer_t(track));
 
       if (track->type == RMFF_TRACK_TYPE_VIDEO) {
         dmx->rvp = (real_video_props_t *)track->mdpr_header.type_specific_data;
@@ -298,7 +296,6 @@ real_reader_c::parse_headers() {
 
 void
 real_reader_c::create_packetizer(int64_t tid) {
-  int i;
   real_demuxer_cptr dmx;
   rmff_track_t *track;
 
@@ -340,6 +337,7 @@ real_reader_c::create_packetizer(int64_t tid) {
       } else if (!strcasecmp(dmx->fourcc, "raac") ||
                  !strcasecmp(dmx->fourcc, "racp")) {
         int profile, channels, sample_rate, output_sample_rate;
+        int detected_profile;
         uint32_t extra_len;
         bool sbr, extra_data_parsed;
 
@@ -377,13 +375,21 @@ real_reader_c::create_packetizer(int64_t tid) {
           dmx->channels = channels;
           dmx->samples_per_second = sample_rate;
         }
+        detected_profile = profile;
         if (sbr)
           profile = AAC_PROFILE_SBR;
-        for (i = 0; i < (int)ti.aac_is_sbr.size(); i++)
-          if ((ti.aac_is_sbr[i] == -1) || (ti.aac_is_sbr[i] == track->id)) {
-            profile = AAC_PROFILE_SBR;
-            break;
-          }
+
+        if ((map_has_key(ti.all_aac_is_sbr, track->id) &&
+             ti.all_aac_is_sbr[track->id]) ||
+            (map_has_key(ti.all_aac_is_sbr, -1) && ti.all_aac_is_sbr[-1]))
+          profile = AAC_PROFILE_SBR;
+
+        if ((-1 != detected_profile) &&
+            ((map_has_key(ti.all_aac_is_sbr, track->id) &&
+              !ti.all_aac_is_sbr[track->id]) ||
+             (map_has_key(ti.all_aac_is_sbr, -1) && !ti.all_aac_is_sbr[-1])))
+          profile = detected_profile;
+
         mxverb(2, PFX "2. profile: %d, channels: %d, sample_rate: "
                "%d, output_sample_rate: %d, sbr: %d\n", profile, channels,
                sample_rate, output_sample_rate, (int)sbr);
@@ -411,6 +417,9 @@ real_reader_c::create_packetizer(int64_t tid) {
         dmx->ref_timecode = -1;
 
       } else {
+        if (!strcasecmp(dmx->fourcc, "COOK"))
+          dmx->cook_audio_fix = true;
+
         ptzr = new ra_packetizer_c(this, dmx->samples_per_second,
                                    dmx->channels, dmx->bits_per_sample,
                                    (dmx->fourcc[0] << 24) |
@@ -508,13 +517,26 @@ real_reader_c::read(generic_packetizer_c *,
     return FILE_STATUS_MOREDATA;
   }
 
+  if (dmx->cook_audio_fix && dmx->first_frame &&
+      ((frame->flags & RMFF_FRAME_FLAG_KEYFRAME) !=
+       RMFF_FRAME_FLAG_KEYFRAME))
+    dmx->force_keyframe_flag = true;
+
+  if (dmx->force_keyframe_flag &&
+      ((frame->flags & RMFF_FRAME_FLAG_KEYFRAME) ==
+       RMFF_FRAME_FLAG_KEYFRAME))
+    dmx->force_keyframe_flag = false;
+
+  if (dmx->force_keyframe_flag)
+    frame->flags |= RMFF_FRAME_FLAG_KEYFRAME;
+
   if (dmx->track->type == RMFF_TRACK_TYPE_VIDEO)
     assemble_video_packet(dmx, frame);
 
   else if (dmx->is_aac) {
     // If the first AAC packet does not start at 0 then let the AAC
     // packetizer adjust its data accordingly.
-    if (dmx->ref_timecode == -1) {
+    if (dmx->first_frame) {
       dmx->ref_timecode = timecode;
       PTZR(dmx->ptzr)->set_displacement_maybe(timecode);
     }
@@ -526,6 +548,8 @@ real_reader_c::read(generic_packetizer_c *,
 
   rmff_release_frame(frame);
 
+  dmx->first_frame = false;
+
   return FILE_STATUS_MOREDATA;
 }
 
@@ -581,7 +605,9 @@ real_reader_c::deliver_audio_frames(real_demuxer_cptr dmx,
            (uint32_t)segment->flags, duration);
     PTZR(dmx->ptzr)->process(new packet_t(segment->data,
                                           dmx->last_timecode, duration,
-                                          (segment->flags & 2) == 2 ? -1 :
+                                          (segment->flags &
+                                           RMFF_FRAME_FLAG_KEYFRAME) ==
+                                          RMFF_FRAME_FLAG_KEYFRAME ? -1 :
                                           dmx->ref_timecode));
     if ((segment->flags & 2) == 2)
       dmx->ref_timecode = dmx->last_timecode;
