@@ -6,7 +6,7 @@
    see the file COPYING for details
    or visit http://www.gnu.org/copyleft/gpl.html
 
-   $Id: cluster_helper.cpp 3154 2005-12-02 14:14:45Z mosu $
+   $Id: cluster_helper.cpp 3455 2007-01-10 13:13:23Z mosu $
 
    The cluster helper groups frames into blocks groups and those
    into clusters, sets the durations, renders the clusters etc.
@@ -14,7 +14,10 @@
    Written by Moritz Bunkus <moritz@bunkus.org>.
 */
 
-#include <assert.h>
+#include "os.h"
+
+#include <cassert>
+#include <limits.h>
 
 #include <vector>
 
@@ -47,6 +50,7 @@ public:
 #define walk_clusters()
 
 cluster_helper_c::cluster_helper_c():
+  min_timecode_in_cluster(-1), max_timecode_in_cluster(-1),
   current_split_point(split_points.begin()) {
 
   cluster_content_size = 0;
@@ -79,7 +83,7 @@ cluster_helper_c::get_cluster() {
 void
 cluster_helper_c::add_packet(packet_cptr packet) {
   ch_contents_t *c;
-  int64_t timecode, additional_size;
+  int64_t timecode, timecode_delay, additional_size;
   int i;
   bool split;
 
@@ -97,17 +101,27 @@ cluster_helper_c::add_packet(packet_cptr packet) {
 
   timecode = get_timecode();
 
+  timecode_delay =
+    (packet->assigned_timecode > max_timecode_in_cluster) ||
+    (-1 == max_timecode_in_cluster) ?
+    packet->assigned_timecode : max_timecode_in_cluster;
+  timecode_delay -=
+    (-1 == min_timecode_in_cluster) ||
+    (packet->assigned_timecode < min_timecode_in_cluster) ?
+    packet->assigned_timecode : min_timecode_in_cluster;
+  timecode_delay = (int64_t)(timecode_delay / timecode_scale);
+
   mxverb(4, "cluster_helper_c::add_packet(): new packet { source " LLD "/%s "
          "timecode: " LLD " duration: " LLD " bref: " LLD " fref: " LLD " "
-         "assigned_timecode: " LLD " }\n",
+         "assigned_timecode: " LLD " timecode_delay: " FMT_TIMECODEN " }\n",
          packet->source->ti.id, packet->source->ti.fname.c_str(),
          packet->timecode, packet->duration, packet->bref, packet->fref,
-         packet->assigned_timecode);
+         packet->assigned_timecode, ARG_TIMECODEN(timecode_delay));
 
   if (clusters.size() == 0)
     add_cluster(new kax_cluster_c());
-  else if ((packet->gap_following && (clusters.back()->packets.size() != 0))
-           ||
+  else if ((SHRT_MAX < timecode_delay) || (SHRT_MIN > timecode_delay) ||
+           (packet->gap_following && (clusters.back()->packets.size() != 0)) ||
            (((packet->assigned_timecode - timecode) > max_ns_per_cluster) &&
             all_references_resolved(clusters.back()))) {
     render();
@@ -196,6 +210,12 @@ cluster_helper_c::add_packet(packet_cptr packet) {
 
   walk_clusters();
 
+  if (packet->assigned_timecode > max_timecode_in_cluster)
+    max_timecode_in_cluster = packet->assigned_timecode;
+  if ((-1 == min_timecode_in_cluster) ||
+      (packet->assigned_timecode < min_timecode_in_cluster))
+    min_timecode_in_cluster = packet->assigned_timecode;
+
   // Render the cluster if it is full (according to my many criteria).
   timecode = get_timecode();
   if ((((packet->assigned_timecode - timecode) > max_ns_per_cluster) ||
@@ -322,6 +342,32 @@ cluster_helper_c::set_duration(render_groups_t *rg) {
     group->SetBlockDuration(RND_TIMECODE_SCALE(block_duration));
 }
 
+bool
+cluster_helper_c::must_duration_be_set(render_groups_t *rg,
+                                       packet_cptr &new_packet) {
+  uint32_t i;
+  int64_t block_duration, def_duration;
+
+  block_duration = 0;
+  for (i = 0; i < rg->durations.size(); i++)
+    block_duration += rg->durations[i];
+  block_duration += new_packet->duration;
+  def_duration = rg->source->get_track_default_duration();
+
+  if (rg->duration_mandatory || new_packet->duration_mandatory) {
+    if ((block_duration == 0) ||
+        ((block_duration > 0) &&
+         (block_duration != ((rg->durations.size() + 1) * def_duration))))
+      return true;
+  } else if ((use_durations || (def_duration > 0)) &&
+             (block_duration > 0) &&
+             (RND_TIMECODE_SCALE(block_duration) !=
+              RND_TIMECODE_SCALE((rg->durations.size() + 1) * def_duration)))
+    return true;
+
+  return false;
+}
+
 /*
   <+Asylum> The chicken and the egg are lying in bed next to each
             other after a good hard shag, the chicken is smoking a
@@ -351,7 +397,8 @@ cluster_helper_c::render_cluster(ch_contents_t *clstr) {
   render_groups_t *render_group;
   bool added_to_cues;
   LacingType lacing_type;
-  BlockBlobType use_simpleblock = hack_engaged(ENGAGE_USE_SIMPLE_BLOCK) ?
+  bool use_simpleblock = hack_engaged(ENGAGE_USE_SIMPLE_BLOCK);
+  BlockBlobType this_block_blob_type, std_block_blob_type = use_simpleblock ?
     BLOCK_BLOB_ALWAYS_SIMPLE : BLOCK_BLOB_NO_SIMPLE;
 
   assert((clstr != NULL) && !clstr->rendered);
@@ -413,14 +460,20 @@ cluster_helper_c::render_cluster(ch_contents_t *clstr) {
 
     if (pack->bref != -1)
       render_group->more_data = false;
+
     if (!render_group->more_data) {
       set_duration(render_group);
-      new_block_group = new KaxBlockBlob(use_simpleblock);
+      render_group->durations.clear();
+      render_group->duration_mandatory = false;
+
+      this_block_blob_type = !use_simpleblock ? std_block_blob_type :
+        must_duration_be_set(render_group, pack) ?
+        BLOCK_BLOB_NO_SIMPLE : BLOCK_BLOB_ALWAYS_SIMPLE;
+
+      new_block_group = new KaxBlockBlob(this_block_blob_type);
       cluster->AddBlockBlob(new_block_group);
       new_block_group->SetParent(*cluster);
       render_group->groups.push_back(new_block_group);
-      render_group->durations.clear();
-      render_group->duration_mandatory = false;
       added_to_cues = false;
     } else
       new_block_group = last_block_group;
@@ -577,6 +630,9 @@ cluster_helper_c::render_cluster(ch_contents_t *clstr) {
 
   free_clusters();
 
+  min_timecode_in_cluster = -1;
+  max_timecode_in_cluster = -1;
+
   return 1;
 }
 
