@@ -6,7 +6,7 @@
    see the file COPYING for details
    or visit http://www.gnu.org/copyleft/gpl.html
 
-   $Id: pr_generic.cpp 3341 2006-11-07 15:11:49Z mosu $
+   $Id: pr_generic.cpp 3421 2007-01-04 08:07:17Z mosu $
 
    functions common for all readers/packetizers
 
@@ -205,6 +205,18 @@ generic_packetizer_c::generic_packetizer_c(generic_reader_c *nreader,
   else if (map_has_key(ti.max_blockadd_ids, -1))
     htrack_max_add_block_ids = ti.max_blockadd_ids[-1];
 
+  // Let's see if the user has specified "AAC is SBR" for this track.
+  if (map_has_key(ti.all_aac_is_sbr, ti.id))
+    ti.aac_is_sbr = ti.all_aac_is_sbr[ti.id] ? 1 : 0;
+  else if (map_has_key(ti.all_aac_is_sbr, -1))
+    ti.aac_is_sbr = ti.all_aac_is_sbr[-1] ? 1 : 0;
+
+  // Let's see if the user has specified a NALU size length for this track.
+  if (map_has_key(ti.nalu_size_lengths, ti.id))
+    ti.nalu_size_length = ti.nalu_size_lengths[ti.id];
+  else if (map_has_key(ti.nalu_size_lengths, -1))
+    ti.nalu_size_length = ti.nalu_size_lengths[-1];
+
   // Set default header values to 'unset'.
   if (!reader->appending)
     hserialno = create_track_number(reader, ti.id);
@@ -924,7 +936,7 @@ void
 generic_packetizer_c::process_deferred_packets() {
   deque<packet_cptr>::iterator packet;
 
-  foreach(packet, deferred_packets)
+  mxforeach(packet, deferred_packets)
     add_packet2(*packet);
   deferred_packets.clear();
 }
@@ -1228,7 +1240,7 @@ generic_reader_c::generic_reader_c(track_info_c &_ti):
   add_all_requested_track_ids(string, languages);
   add_all_requested_track_ids(string, sub_charsets);
   add_all_requested_track_ids(string, all_tags);
-  add_all_requested_track_ids2(aac_is_sbr);
+  add_all_requested_track_ids(bool, all_aac_is_sbr);
   add_all_requested_track_ids(int64_t, packet_delays);
   add_all_requested_track_ids(compression_method_e, compression_list);
   add_all_requested_track_ids(string, track_names);
@@ -1306,7 +1318,7 @@ generic_reader_c::set_timecode_offset(int64_t offset) {
   vector<generic_packetizer_c *>::const_iterator it;
 
   max_timecode_seen = offset;
-  foreach(it, reader_packetizers)
+  mxforeach(it, reader_packetizers)
     (*it)->correction_timecode_offset = offset;
 }
 
@@ -1314,7 +1326,7 @@ void
 generic_reader_c::set_headers() {
   vector<generic_packetizer_c *>::const_iterator it;
 
-  foreach(it, reader_packetizers)
+  mxforeach(it, reader_packetizers)
     (*it)->set_headers();
 }
 
@@ -1322,7 +1334,7 @@ void
 generic_reader_c::set_headers_for_track(int64_t tid) {
   vector<generic_packetizer_c *>::const_iterator it;
 
-  foreach(it, reader_packetizers)
+  mxforeach(it, reader_packetizers)
     if ((*it)->ti.id == tid) {
       (*it)->set_headers();
       break;
@@ -1378,7 +1390,7 @@ generic_reader_c::get_queued_bytes() {
   int64_t bytes;
 
   bytes = 0;
-  foreach(it, reader_packetizers)
+  mxforeach(it, reader_packetizers)
     bytes += (*it)->get_queued_bytes();
 
   return bytes;
@@ -1388,7 +1400,7 @@ void
 generic_reader_c::flush_packetizers() {
   vector<generic_packetizer_c *>::const_iterator it;
 
-  foreach(it, reader_packetizers)
+  mxforeach(it, reader_packetizers)
     (*it)->flush();
 }
 
@@ -1413,10 +1425,12 @@ track_info_c::track_info_c():
   cues(CUE_STRATEGY_UNSPECIFIED),
   default_track(false),
   tags(NULL),
+  aac_is_sbr(-1),
   packet_delay(0),
   compression(COMPRESSION_UNSPECIFIED),
   pixel_cropping_specified(false),
   stereo_mode(STEREO_MODE_UNSPECIFIED),
+  nalu_size_length(0),
   no_chapters(false),
   no_attachments(false),
   no_tags(false),
@@ -1488,6 +1502,7 @@ track_info_c::operator =(const track_info_c &src) {
   else
     tags = NULL;
 
+  all_aac_is_sbr = src.all_aac_is_sbr;
   aac_is_sbr = src.aac_is_sbr;
 
   packet_delay = src.packet_delay;
@@ -1509,6 +1524,9 @@ track_info_c::operator =(const track_info_c &src) {
   stereo_mode_list = src.stereo_mode_list;
   stereo_mode = src.stereo_mode;
 
+  nalu_size_lengths = src.nalu_size_lengths;
+  nalu_size_length = src.nalu_size_length;
+
   no_chapters = src.no_chapters;
   no_attachments = src.no_attachments;
   no_tags = src.no_tags;
