@@ -6,7 +6,7 @@
    see the file COPYING for details
    or visit http://www.gnu.org/copyleft/gpl.html
 
-   $Id: mmg.h 3344 2006-11-08 21:45:18Z mosu $
+   $Id: mmg.h 3463 2007-01-12 16:02:16Z mosu $
 
    main stuff - declarations
 
@@ -23,6 +23,7 @@
 
 #include "wx/string.h"
 #include "wx/app.h"
+#include "wx/combobox.h"
 
 #include "ebml/EbmlUnicodeString.h"
 
@@ -56,20 +57,25 @@ struct mmg_track_t {
   wxString ctype;
   bool enabled, display_dimensions_selected;
 
-  bool default_track, aac_is_sbr;
+  bool default_track, aac_is_sbr, aac_is_sbr_detected;
   bool track_name_was_present;
   wxString language, track_name, cues, delay, stretch, sub_charset;
-  wxString tags, fourcc, aspect_ratio, compression, timecodes;
+  wxString tags, fourcc, aspect_ratio, compression, timecodes, fps;
+  int nalu_size_length;
   wxString dwidth, dheight;
   int stereo_mode;
 
+  wxString user_defined;
+
   bool appending;
 
   mmg_track_t():
     type(0), id(0), source(0),
     enabled(false), display_dimensions_selected(false),
-    default_track(false), aac_is_sbr(false), track_name_was_present(false),
+    default_track(false), aac_is_sbr(false), aac_is_sbr_detected(false),
+    track_name_was_present(false),
     language(wxT("und")), cues(wxT("default")), sub_charset(wxT("default")),
+    nalu_size_length(2),
     stereo_mode(0),
     appending(false) {};
 };
@@ -130,6 +136,8 @@ wxString create_track_order(bool all);
 
 int default_track_checked(char type);
 
+void set_combobox_selection(wxComboBox *cb, const wxString wanted);
+
 void wxdie(const wxString &errmsg);
 
 #if defined(SYS_WINDOWS)
